/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: afunix.c,v 1.6 1999/04/08 17:50:29 bazsi Exp $
 *
 ***************************************************************************/

#include "afunix.h"
#include "format.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <unistd.h>
#include <errno.h>

#include "afunix.c.x"

int open_unix_socket(struct sockaddr_un *sun, int proto, int *sockfd)
{
	int socktype = (proto == AFUNIX_PROTO_DGRAM ? SOCK_DGRAM : SOCK_STREAM);
	int fd;
	
	fd = socket(AF_UNIX, socktype, 0);
	if (fd != -1) {
		unlink(sun->sun_path);
		errno = 0;
		if (bind(fd, (struct sockaddr *) sun, sizeof(struct sockaddr_un)) == -1) {
			werror("cannot bind af_unix socket to %z (%z)\n", sun->sun_path, strerror(errno));
			close(fd);
			return 0;
		}
		chmod(sun->sun_path, 0666);
		*sockfd = fd;
		return 1;
	}
	/* indicate failure */
	return 0;
}


/* CLASS:
     (class
       (name afunix_source_connection)
       (super log_handler_pipe)
       (vars
         (afunix object afunix_source)
         (fd simple int)
         (src object io_fd)
         (next_connection object afunix_source_connection)))
*/

static int do_init_afunix_source_connection(struct log_handler *c)
{
	CAST(afunix_source_connection, self, c);
	
	self->src = io_read(self->afunix->backend, self->fd, 
			make_read_line(make_log_reader(self->super.next)),
			NULL);
	REMEMBER_RESOURCE(self->afunix->resources, &self->src->super.super);
	return 0;
}

struct afunix_source_connection *make_afunix_source_connection(int fd, struct afunix_source *afunix)
{
	NEW(afunix_source_connection, self);

	self->super.super.init = do_init_afunix_source_connection;
	self->super.next = afunix->super.super.next;
	self->afunix = afunix;
	self->fd = fd;
	return self;
}

/* CLASS:
     (class
       (name afunix_accept_callback)
       (super fd_listen_callback)
       (vars
         (afunix object afunix_source)))
*/

static int do_accept_unix_connection(struct fd_listen_callback *c, int fd, size_t salen, struct sockaddr *sa)
{
	CAST(afunix_accept_callback, self, c);
	struct afunix_source_connection *conn;
	
	conn = make_afunix_source_connection(fd, self->afunix);
	return LOG_HANDLER_INIT(conn);
}

struct fd_listen_callback *make_afunix_accept_callback(struct afunix_source *afunix)
{
	NEW(afunix_accept_callback, self);
	
	self->super.f = do_accept_unix_connection;
	self->afunix = afunix;
	return &self->super;
}

/* CLASS:
     (class
       (name afunix_source)
       (super log_source_driver)
       (vars
         (backend object io_backend)
         (src object io_fd)
         (listen object listen_fd)
	 (resources object resource_list)
         (name string)
         (proto simple int)))
*/

int do_init_afunix_source(struct log_handler *c)
{
	CAST(afunix_source, self, c);
	int fd;
	struct sockaddr_un sun;
	
	sun.sun_family = AF_UNIX;
	strncpy(sun.sun_path, self->name->data, sizeof(sun.sun_path));
	if (open_unix_socket(&sun, self->proto, &fd)) {
		switch (self->proto) {
		case AFUNIX_PROTO_STREAM:
			if ((self->listen = io_listen(self->backend, fd, 
					make_afunix_accept_callback(self))))
				return ST_OK | ST_GOON;
			break;
		case AFUNIX_PROTO_DGRAM:
			self->src = io_read(self->backend, fd, 
				make_read_line(make_log_reader((struct log_handler *) self->super.super.next)), 
				NULL);
			return ST_OK | ST_GOON;
		}
	}
	return ST_FAIL | ST_QUIT;
}

struct log_source_driver *make_afunix_source(const char *name, int proto, struct io_backend *backend)
{
        NEW(afunix_source, self);

        self->super.super.super.init = do_init_afunix_source;
        self->backend = backend;
	self->resources = empty_resource_list();
        self->name = c_format("%z%c", name, 0);
        self->name->length--;
        self->proto = proto;
        return &self->super;
}
